// The xMule Project - A Peer-2-Peer File Sharing Program
//
// Copyright (C) 2003-2006 Theodore R. Smith ( hopeseekr@gmail.com / http://www.xmule.ws/ )
// Copyright (C) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of Version 2 of the GNU General Public
// License as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA


#ifdef PRECOMP
    #include "xmule-headers.h"
#endif

#include "PPgDirectories.h"
#include "otherfunctions.h"
#include "PPgDirectories.h"
#include "Preferences.h"          // CPreferences
#include "SharedFileList.h"
#include "wintypes.h"
#include "xmule.h"

#include <DynPrefs/DynPrefs.h>    // DynamicPreferences
#include <wx/checkbox.h>          // wxCheckBox
#include <wx/notebook.h>          // wxNotebook
#include <wx/slider.h>
#include <wx/textctrl.h>          // wxTextCtrl
#include <wx/treectrl.h>          // wxTreeCtrl
#include <wx/dirdlg.h>            // wxDirSelector
#include <wx/filedlg.h>           // wxFileSelector
#include <wx/xrc/xmlres.h>        // wxXmlResource

// Mario Sergio Fujikawa Ferreira <lioux@FreeBSD.org>
// to detect if this is a *BSD system
#if defined(HAVE_SYS_PARAM_H)
#include <sys/param.h>
#endif
#ifndef MAX_PATH
#if (defined(BSD) && (BSD >= 199103))
#define MAX_PATH MAXPATHLEN
#else
#define MAX_PATH 2048
#endif
#endif

#define GetDlgItem(x,clas) XRCCTRL(*this,#x,clas)
#define IsDlgButtonChecked(x) XRCCTRL(*this,#x,wxCheckBox)->GetValue()
#define CheckDlgButton(x,y) XRCCTRL(*this,#x,wxCheckBox)->SetValue(y)

// CPPgDirectories Panel

BEGIN_EVENT_TABLE(CPPgDirectories, wxPanel)
    EVT_BUTTON(XRCID("IDC_SELTEMPDIR"), CPPgDirectories::OnBnClickedSeltempdir)
    EVT_BUTTON(XRCID("IDC_SELINCDIR"), CPPgDirectories::OnBnClickedSelincdir)
    EVT_BUTTON(XRCID("IDC_BROWSEV"), CPPgDirectories::BrowseVideoplayer)
END_EVENT_TABLE()

CPPgDirectories::CPPgDirectories(wxWindow *parent) : wxPanel(parent, CPPgDirectories::IDD)
{
    wxNotebook* book = (wxNotebook *) parent;
    wxPanel *page1 = wxXmlResource::Get()->LoadPanel(this, wxT("DLG_PPG_DIRECTORIES"));
    book->AddPage(this, _("Directories"));
    SetSize(page1->GetSize().GetWidth(), page1->GetSize().GetHeight() + 40);
    // then replace the existing tree control with our own
    wxTreeCtrl *oldtree = GetDlgItem(IDC_SHARESELECTOR, wxTreeCtrl);
    wxPoint pos = oldtree->GetPosition();
    wxSize size = oldtree->GetSize();
    // then replace it with our own
    oldtree->Destroy();
    Localize();
}

void CPPgDirectories::LoadSettings()
{
    GetDlgItem(IDC_INCFILES, wxTextCtrl)->SetValue(DynPrefs::Get<wxString>("incoming-directory"));
    GetDlgItem(IDC_TEMPFILES, wxTextCtrl)->SetValue(DynPrefs::Get<wxString>("temp-directory"));

    GetDlgItem(IDC_VIDEOPLAYER, wxTextCtrl)->SetValue(wxString(app_prefs->prefs->VideoPlayer, *wxConvCurrent));
    if (app_prefs->prefs->moviePreviewBackup)
    {
        CheckDlgButton(IDC_VIDEOBACKUP, 1);
    }
    else
    {
        CheckDlgButton(IDC_VIDEOBACKUP, 0);
    }
}

bool CPPgDirectories::SelectDir(wxString& outdir, wxString const& TitleText)
{
    wxString str = wxDirSelector(TitleText);
    if (str.IsEmpty())
    {
        return false;
    }
    outdir = str;
    return true;
}

void CPPgDirectories::OnBnClickedSelincdir(wxCommandEvent& e)
{
    wxString NewDir;
    if (SelectDir(NewDir, _("Choose a folder for incoming files")))
    {
        GetDlgItem(IDC_INCFILES, wxTextCtrl)->SetValue(NewDir);
    }
}

void CPPgDirectories::OnBnClickedSeltempdir(wxCommandEvent& e)
{
    wxString NewDir;
    if (SelectDir(NewDir, _("Choose a folder for temp files")))
    {
        GetDlgItem(IDC_TEMPFILES, wxTextCtrl)->SetValue(NewDir);
    }
}

void CPPgDirectories::BrowseVideoplayer(wxCommandEvent &e)
{
    wxString str = wxFileSelector(_("Browse for videoplayer"), wxT(""), wxT(""), wxT(""), _("Executable (*)|*||"));
    if (!str.IsEmpty())
    {
        GetDlgItem(IDC_VIDEOPLAYER, wxTextCtrl)->SetValue(str);
    }
}

bool CPPgDirectories::OnApply()
{
    wxString buffer;
    bool IncomingChanged = false; // Will be set true if Incoming directory changed:
    bool TempChanged = false; // Will be set true if Temp directory changed:
    if (GetDlgItem(IDC_INCFILES, wxTextCtrl)->GetValue().Length())
    {
        buffer = GetDlgItem(IDC_INCFILES, wxTextCtrl)->GetValue();
        if (DynPrefs::Get<wxString>("incoming-directory").Cmp(buffer) != 0)
        {
            IncomingChanged = true;
            DynPrefs::Add("incoming-directory", buffer);
        }
    }
    if (GetDlgItem(IDC_TEMPFILES, wxTextCtrl)->GetValue().Length())
    {
        buffer = GetDlgItem(IDC_TEMPFILES, wxTextCtrl)->GetValue();
        if (DynPrefs::Get<wxString>("temp-directory").Cmp(buffer) != 0)
        {
            TempChanged = true;
            DynPrefs::Add("temp-directory", buffer);
        }
    }
    buffer = GetDlgItem(IDC_VIDEOPLAYER, wxTextCtrl)->GetValue();
    strcpy(app_prefs->prefs->VideoPlayer, buffer.mb_str());
    app_prefs->prefs->moviePreviewBackup = IsDlgButtonChecked(IDC_VIDEOBACKUP);
    app_prefs->shareddir_list.Clear();

    if (IncomingChanged || TempChanged)
    {
        theApp.sharedfiles->Reload(true, false);
    }
    return true;
}

void CPPgDirectories::Localize()
{
    GetDlgItem(IDC_INCOMING_FRM, wxControl)->SetLabel(GetResString(IDS_PW_INCOMING));
    GetDlgItem(IDC_TEMP_FRM, wxControl)->SetLabel(GetResString(IDS_PW_TEMP));
    GetDlgItem(IDC_SELINCDIR, wxControl)->SetLabel(GetResString(IDS_PW_BROWSE));
    GetDlgItem(IDC_SELTEMPDIR, wxControl)->SetLabel(GetResString(IDS_PW_BROWSE));
    GetDlgItem(IDC_BROWSEV, wxControl)->SetLabel(GetResString(IDS_PW_BROWSE));
    GetDlgItem(IDC_SHARED_FRM, wxControl)->SetLabel(GetResString(IDS_PW_SHARED));
    GetDlgItem(IDC_STATICVIDEOPLAYER, wxControl)->SetLabel(GetResString(IDS_PW_VIDEOPLAYER));
    GetDlgItem(IDC_VIDEOBACKUP, wxControl)->SetLabel(GetResString(IDS_VIDEOBACKUP));
    GetDlgItem(IDC_STATIC_EMPTY, wxControl)->SetLabel(GetResString(IDS_STATIC_EMPTY));
}
