// The xMule Project - A Peer-2-Peer File Sharing Program
//
// Copyright (C) 2003-2006 Theodore R. Smith ( hopeseekr@gmail.com / http://www.xmule.ws/ )
// Copyright (C) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of Version 2 of the GNU General Public
// License as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

#ifdef PRECOMP
    #include "xmule-headers.h"
#endif

#include "otherfunctions.h"
#include "PartFile.h"
#include "Preferences.h"          // CPreferences::GetAllinCat
#include "resource.h"
#include "wintypes.h"             // x::BYTE, x::DWORD, x::WORD
#include "xmule.h"

#include <iostream>               // std::cout
#include <map>                    // std::map
#include <iomanip>
#include <sstream>
#include <stdint.h>               // uint32_t
#include <wx/utils.h>
#include <wx/file.h>              // wxFile

#include "resource.h"                       // Needed for strings' IDs
#include "strings.en"                       // Needed for strings

using std::cout;
using std::endl;
using std::setw;
using std::hex;
using std::setprecision;
using std::setfill;

#define itemsof(x) (sizeof(x)/sizeof(x[0]))

wxString GetResString(unsigned int uStringID)
{
    static std::map<unsigned int, wxString> msgs;

    if (msgs.empty())
    {
        for (unsigned int a = 0; a < itemsof(_resStrings); ++a)
        {
            msgs[_resStrings[a].id] = wxString(_resStrings[a].resstr, *wxConvCurrent);
        }        
    }
    return msgs[uStringID];
}

// Base chars for encode an decode functions
static x::BYTE base16Chars[17] = "0123456789ABCDEF";

static x::BYTE base32Chars[33] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ234567";

#define BASE16_LOOKUP_MAX 23
static x::BYTE base16Lookup[BASE16_LOOKUP_MAX][2] =
{
    {
        '0', 0x0
    }
    ,
    {
        '1', 0x1
    }
    ,
    {
        '2', 0x2
    }
    ,
    {
        '3', 0x3
    }
    ,
    {
        '4', 0x4
    }
    ,
    {
        '5', 0x5
    }
    ,
    {
        '6', 0x6
    }
    ,
    {
        '7', 0x7
    }
    ,
    {
        '8', 0x8
    }
    ,
    {
        '9', 0x9
    }
    ,
    {
        ':', 0x9
    }
    ,
    {
        ';', 0x9
    }
    ,
    {
        '<', 0x9
    }
    ,
    {
        '=', 0x9
    }
    ,
    {
        '>', 0x9
    }
    ,
    {
        '?', 0x9
    }
    ,
    {
        '@', 0x9
    }
    ,
    {
        'A', 0xA
    }
    ,
    {
        'B', 0xB
    }
    ,
    {
        'C', 0xC
    }
    ,
    {
        'D', 0xD
    }
    ,
    {
        'E', 0xE
    }
    ,
    {
        'F', 0xF
    }
}

;

wxString CastItoXBytes(uint64_t count)
{
    std::ostringstream internalbuffer;

    if (count < 1024)
    {
        internalbuffer << setprecision(0) << (float)count << " " << GetResString(IDS_BYTES).mb_str(*wxConvCurrent);
    }
    else if(count < 1048576)
    {
        internalbuffer << setprecision(0) << (float)count/1024 << " " << GetResString(IDS_KBYTES).mb_str(*wxConvCurrent);
    }
    else if(count < 1073741824)
    {
        internalbuffer << setprecision(2) << (float)count/1048576 << " " << GetResString(IDS_MBYTES).mb_str(*wxConvCurrent);
    }
    else if(count < 1099511627776LL)
    {
        internalbuffer << setprecision(2) << (float)count/1073741824 << " " << GetResString(IDS_GBYTES).mb_str(*wxConvCurrent);
    }
    else
    {
        internalbuffer << setprecision(3) << (float)count/1099511627776LL << " " << GetResString(IDS_TBYTES).mb_str(*wxConvCurrent);
    }

    return wxString(internalbuffer.str().c_str(), *wxConvCurrent);
}

wxString CastItoIShort(uint64_t count)
{
    std::ostringstream internalbuffer;

    if (count < 1000)
    {
        internalbuffer << count;
    }
    else if(count < 1000000)
    {
        internalbuffer << setprecision(0) << static_cast<float>(count / 1000) << GetResString(IDS_KILO).mb_str(*wxConvCurrent);
    }
    else if(count < 1000000000)
    {
        internalbuffer << setprecision(2) << static_cast<float>(count / 1000000) << GetResString(IDS_MEGA).mb_str(*wxConvCurrent);
    }
    else if(count < 1000000000000LL)
    {
        internalbuffer << setprecision(2) << static_cast<float>(count / 1000000000LL) << GetResString(IDS_GIGA).mb_str(*wxConvCurrent);
    }
    else if(count < 1000000000000000LL)
    {
        internalbuffer << setprecision(2) << static_cast<float>(count / 1000000000000LL) << GetResString(IDS_TERRA).mb_str(*wxConvCurrent);
    }

    return wxString(internalbuffer.str().c_str(), *wxConvCurrent);
}

wxString CastSecondsToHM(int32_t count)
{
    std::ostringstream internalbuffer;

    if (count < 0)
    {
        internalbuffer << "?";
    }
    else if(count < 60)
    {
        internalbuffer << count << " " << GetResString(IDS_SECS).mb_str(*wxConvCurrent);
    }
    else if(count < 3600)
    {
        internalbuffer << count / 60 << ":" << LeadingZero(count - (count/60) *60).mb_str(*wxConvCurrent) << " " << GetResString(IDS_MINS).mb_str(*wxConvCurrent);
    }
    else if(count < 86400)
    {
        internalbuffer << count  /3600 << ":" << LeadingZero((count - (count/3600) *3600) /60).mb_str(*wxConvCurrent) << " " << GetResString(IDS_HOURS).mb_str(*wxConvCurrent);
    }
    else
    {
        internalbuffer << count / 86400 << " " << GetResString(IDS_DAYS).mb_str(*wxConvCurrent) << " " << (count - (count/86400) *86400) /3600 << " " << GetResString(IDS_HOURS).mb_str(*wxConvCurrent);
    }

    return wxString(internalbuffer.str().c_str(), *wxConvCurrent);
}

wxString LeadingZero(uint32_t units)
{
    std::ostringstream internalbuffer;
    if (units < 10)
    {
        internalbuffer << "0" << units;
    }
    else
    {
        internalbuffer << units;
    }

    return wxString(internalbuffer.str().c_str(), *wxConvCurrent);
}

void ShellOpenFile(wxString name)
{
    //ShellExecute(NULL, "open", name, NULL, NULL, SW_SHOW);
    cout << "TODO: shellopen" << endl;
}

namespace
{
    bool IsHexDigit(int c)
    {
        switch (c)
        {
        case '0': return true;
        case '1': return true;
        case '2': return true;
        case '3': return true;
        case '4': return true;
        case '5': return true;
        case '6': return true;
        case '7': return true;
        case '8': return true;
        case '9': return true;
        case 'A': return true;
        case 'B': return true;
        case 'C': return true;
        case 'D': return true;
        case 'E': return true;
        case 'F': return true;
        case 'a': return true;
        case 'b': return true;
        case 'c': return true;
        case 'd': return true;
        case 'e': return true;
        case 'f': return true;
        default: return false;
        }
    }
}

void URLDecode(wxString &result, const char *buff)
{
    int buflen = (int) strlen(buff);
    int x;
    int y;
    // length of buff2 will be less or equal to length of buff:
    char *buff2 = nstrdup(buff);
    for (x = 0, y = 0 ; x < buflen ;++x)
    {
        if (buff[x] == '%' &&x + 2 < buflen &&IsHexDigit(buff[x + 1]) &&IsHexDigit(buff[x + 2]))
        {
            char hexstr[3];
            // Copy the two BYTEs following the %
            strncpy(hexstr, &buff[x + 1], 2);
            // Skip over the hex
            x = x + 2;
            // Convert the hex to ASCII
            buff2[y++] = (unsigned char) strtoul(hexstr, NULL, 16);
        }
        else
        {
            buff2[y++] = buff[x];
            break;
        }
    }
    result = wxString(buff2, *wxConvCurrent);
    delete[] buff2;
}

wxString URLEncode(wxString sIn)
{
    wxString sOut;
    const int nLen = sIn.Length() + 1;
    register wxChar* pOutTmp = NULL;
    wxChar* pOutBuf = NULL;
    register wxChar* pInTmp = NULL;
    //GetData(); //GetBuffer(nLen);:
    wxChar* pInBuf = sIn.GetWriteBuf(nLen);
    //alloc out buffer
    //GetData(); //GetBuffer(nLen  * 3 - 2);//new x::BYTE [nLen  * 3];:
    pOutBuf = sOut.GetWriteBuf(nLen *3 - 2);
    if (pOutBuf)
    {
        pInTmp = pInBuf;
        pOutTmp = pOutBuf;
        // do encoding
        while ( *pInTmp)
        {
            if (isalnum( *pInTmp))
            *pOutTmp++= *pInTmp;
            else
            if (isspace( *pInTmp))
            *pOutTmp++= wxT('+');
            else
            {
                *pOutTmp++= wxT('%');
                *pOutTmp++= toHex( *pInTmp >> 4);
                *pOutTmp++= toHex( *pInTmp%16);
            }
            pInTmp++;
        }
        *pOutTmp = wxT('\0');
        sOut = pOutBuf;
        //delete [] pOutBuf;
        //Out.ReleaseBuffer();
        sOut.UngetWriteBuf();
    }
    //sIn.ReleaseBuffer();
    sIn.UngetWriteBuf();
    return sOut;
}

inline unsigned char toHex(const unsigned char& x)
{
    return x > 9 ? x + 55: x + 48;
}

int GetFTIDbyName(wxString infile)
{
    int i;
    wxString ext;
    ext = infile.Trim();
    i = ext.Len();
    while (i > 0)
    {
        i--;
        if (ext[i] == wxT('.'))
        {
            i = - i;
        }
    }
    if (i >= 0)
    {
        // Any:
        return 0;
    }
    else
    {
        i = - i;
        ext = ext.Mid(i + 1, ext.Len() - i - 1) .MakeLower();
        // Type Archive
        if (ext == wxT("gz") ||ext == wxT("zip") ||ext == wxT("ace") ||ext == wxT("rar") ||ext == wxT("arj") || ext == wxT("tar"))
        {
            return 1;
        }
        // Type Audio
        else if(ext == wxT("mpc") ||ext == wxT("mp4") ||ext == wxT("aac") ||ext == wxT("ape") ||ext == wxT("mp3") || ext == wxT("mp2") ||ext == wxT("wav") ||ext == wxT("au") ||ext == wxT("ogg") || ext == wxT("wma") || ext == wxT("mid"))
        {
            return 2;
        }
        // Type Picture
        else if(ext == wxT("jpg") ||ext == wxT("jpeg") ||ext == wxT("bmp") ||ext == wxT("gif") ||ext == wxT("tif") || ext == wxT("png"))
        {
            return 3;
        }
        // Type CD/DVD Image
        else if(ext == wxT("ccd") || ext == wxT("sub") || ext == wxT("cue") || ext == wxT("bin") || ext == wxT("iso") || ext == wxT("nrg") || ext == wxT("img") || ext == wxT("bwa") || ext == wxT("bwi") || ext == wxT("bws") || ext == wxT("bwt") || ext == wxT("mds") || ext == wxT("mdf"))
        {
            return 4;
        }
        // Type Program
        else if(ext == wxT("exe") || ext == wxT("com"))
        {
            return 5;
        }
        // Type Video
        else if(ext == wxT("avi") || ext == wxT("mpg") || ext == wxT("mpeg") || ext == wxT("ram") || ext == wxT("rm") || ext == wxT("asf") || ext == wxT("vob") || ext == wxT("divx") || ext == wxT("vivo") || ext == wxT("ogm") || ext == wxT("mov") || ext == wxT("wmv"))
        {
            return 6;
        }
    }
    // Type Any
    return 0;
}

// Returns the Typename, examining the extention of the given filename
wxString GetFiletypeByName(wxString infile)
{
    wxString ext;
    // For what is this ??? PLK
    if (infile.Find(wxT('.'), true) == - 1) return GetResString(IDS_SEARCH_ANY);
    // ext == extension of the filename (Search Result) PLK
    ext = infile.Right(infile.Length() - infile.Find('.', true)) .MakeLower();
    // Type Audio
    if (ext == wxT(".mpc") || ext == wxT(".mp4") || ext == wxT(".aac") || ext == wxT(".ape") || ext == wxT(".mp3") || ext == wxT(".mp2") || ext == wxT(".wav") || ext == wxT(".au") || ext == wxT(".ogg") || ext == wxT(".wma") || ext == wxT(".mid"))
    {
        return GetResString(IDS_SEARCH_AUDIO);
    }
    // Type Picture
    if (ext == wxT(".jpg") || ext == wxT(".jpeg") || ext == wxT(".bmp") || ext == wxT(".gif") || ext == wxT(".tif") || ext == wxT(".png"))
    {
        return GetResString(IDS_SEARCH_PICS);
    }
    // Type Video
    if (ext == wxT(".avi") || ext == wxT(".mpg") || ext == wxT(".mpeg") || ext == wxT(".ram") || ext == wxT(".rm") || ext == wxT(".asf") || ext == wxT(".vob") || ext == wxT(".divx") || ext == wxT(".vivo") || ext == wxT(".ogm") || ext == wxT(".mov") || ext == wxT(".wmv"))
    {
        return GetResString(IDS_SEARCH_VIDEO);
    }
    // Type Archive
    if (ext == wxT(".gz") || ext == wxT(".zip") || ext == wxT(".ace") || ext == wxT(".rar") || ext == wxT(".tar"))
    {
        return GetResString(IDS_SEARCH_ARC);
    }
    // Type Program
    if (ext == wxT(".exe") || ext == wxT(".com"))
    {
        return GetResString(IDS_SEARCH_PRG);
    }
    // Type CD/DVD Image
    if (ext == wxT(".ccd") || ext == wxT(".sub") || ext == wxT(".cue") || ext == wxT(".bin") || ext == wxT(".iso") || ext == wxT(".nrg") || ext == wxT(".img") || ext == wxT(".bwa") || ext == wxT(".bwi") || ext == wxT(".bws") || ext == wxT(".bwt") || ext == wxT(".mds") || ext == wxT(".mdf"))
    {
        return GetResString(IDS_SEARCH_CDIMG);
    }
    // Type Any
    return GetResString(IDS_SEARCH_ANY);
}

wxString MakeStringEscaped(wxString in)
{
    in.Replace(wxT("&"), wxT("&&"));
    return in;
}

bool Ask4RegFix(bool checkOnly)
{
}

int GetMaxConnections()
{
    return - 1;
}

wxString GetRateString(uint16_t rate)
{
    switch (rate)
    {
    case 0:
        return GetResString(IDS_CMT_NOTRATED);
        break;
    case 1:
        return GetResString(IDS_CMT_FAKE);
        break;
    case 2:
        return GetResString(IDS_CMT_POOR);
        break;
    case 3:
        return GetResString(IDS_CMT_GOOD);
        break;
    case 4:
        return GetResString(IDS_CMT_FAIR);
        break;
    case 5:
        return GetResString(IDS_CMT_EXCELLENT);
        break;
    }
    return GetResString(IDS_CMT_NOTRATED);
}

// Returns a BASE32 encoded x::BYTE array
//
// [In]
//   buffer: Pointer to x::BYTE array
//   bufLen: Lenght of buffer array
//
// [Return]
//   wxString object with BASE32 encoded x::BYTE array
wxString EncodeBase32(const unsigned char *buffer, unsigned int bufLen)
{
    wxString Base32Buff;
    unsigned int i, index;
    unsigned char word;

    for (i = 0, index = 0 ; i < bufLen ;)
    {
        // Is the current word going to span a x::BYTE boundary?
        if (index > 3)
        {
            word = (buffer[i] & (0xFF >> index));
            index = (index + 5) % 8;
            word <<= index;

            if (i < bufLen - 1)
            {
                word |= buffer[i + 1] >> (8 - index);
            }

            i++;
        }
        else
        {
            word = (buffer[i] >> (8 - (index + 5))) &0x1F;
            index = (index + 5) % 8;
            if (index == 0)
            {
                i++;
            }
        }

        Base32Buff += (char)base32Chars[word];
    }

    return Base32Buff;
}

// Returns a BASE16 encoded x::BYTE array
//
// [In]
//   buffer: Pointer to x::BYTE array
//   bufLen: Lenght of buffer array
//
// [Return]
//   wxString object with BASE16 encoded x::BYTE array
wxString EncodeBase16(const unsigned char *buffer, unsigned int bufLen)
{
    wxString Base16Buff;

    for (unsigned int i = 0; i < bufLen; ++i)
    {
        Base16Buff += base16Chars[buffer[i] >> 4];
        Base16Buff += base16Chars[buffer[i] &0xf];
    }

    return Base16Buff;
}

// Decodes a BASE16 string into a x::BYTE array
//
// [In]
//   base16Buffer: String containing BASE16
//   base16BufLen: Lenght BASE16 coded string's length
//
// [Out]
//   buffer: x::BYTE array containing decoded string
void DecodeBase16(const char *base16Buffer, unsigned int base16BufLen, x::BYTE *buffer)
{
    memset(buffer, 0, DecodeLengthBase16(base16BufLen));
    for (unsigned int i = 0 ; i < base16BufLen ; i++)
    {
        int lookup = toupper(base16Buffer[i]) - '0';
        // Check to make sure that the given word falls inside a valid range
        x::BYTE word = 0;
        if (lookup < 0 || lookup >= BASE16_LOOKUP_MAX)
        word = 0xFF;
        else
        word = base16Lookup[lookup][1];
        if (i % 2 == 0)
        {
            buffer[i/2] = word << 4;
        }
        else
        {
            buffer[ (i - 1) /2] |= word;
        }
    }
}

// Calculates length to decode from BASE16
//
// [In]
//   base16Length: Actual length of BASE16 string
//
// [Return]
//   New length of x::BYTE array decoded
int DecodeLengthBase16(int base16Length)
{
    return base16Length / 2;
}

void RunURL(CAbstractFile *file, wxString urlpattern)
{
}

wxString GetCatTitle(int catid)
{
    switch (catid)
    {
        case 0: return GetResString(IDS_ALL);
        case 1: return GetResString(IDS_ALLOTHERS);
        case 2: return GetResString(IDS_STATUS_NOTCOMPLETED);
        case 3: return GetResString(IDS_DL_TRANSFCOMPL);
        case 4: return GetResString(IDS_WAITING);
        case 5: return GetResString(IDS_DOWNLOADING);
        case 6: return GetResString(IDS_ERRORLIKE);
        case 7: return GetResString(IDS_PAUSED);
        case 8: return GetResString(IDS_STOPPED);
        case 9: return GetResString(IDS_VIDEO);
        case 10: return GetResString(IDS_AUDIO);
        case 11: return GetResString(IDS_SEARCH_ARC);
        case 12: return GetResString(IDS_SEARCH_CDIMG);
        case 13: return GetResString(IDS_SEARCH_PICS);
        case 14: return GetResString(IDS_TEXT);
    }
    return wxT("?");
}

bool CheckShowItemInGivenCat(CPartFile *file, int inCategory)
{
    // easy normal cases
    bool IsInCat;
    bool IsNotFiltered = true;
    IsInCat = ((inCategory == 0) || (inCategory > 0 &&inCategory == file->GetCategory()));
    if (theApp.glob_prefs->GetAllcatType() > 0)
    switch (theApp.glob_prefs->GetAllcatType())
    {
    case 1:
        {
            IsNotFiltered = ((file->GetCategory() == 0) || (inCategory > 0));
            break;
        }
    case 2:
        {
            IsNotFiltered = (file->IsPartFile());
            break;
        }
    case 3:
        {
            IsNotFiltered = (!file->IsPartFile());
            break;
        }
    case 4:
        {
            IsNotFiltered = ((file->GetStatus() == PS_READY || file->GetStatus() == PS_EMPTY) &&file->GetTransferingSrcCount() == 0);
            break;
        }
    case 5:
        {
            IsNotFiltered = ((file->GetStatus() == PS_READY || file->GetStatus() == PS_EMPTY) &&file->GetTransferingSrcCount() > 0);
            break;
        }
    case 6:
        {
            IsNotFiltered = (file->GetStatus() == PS_ERROR);
            break;
        }
    case 7:
        {
            IsNotFiltered = ((file->GetStatus() == PS_PAUSED) && (!file->IsStopped()));
            break;
        }
    case 8:
        {
            IsNotFiltered = file->IsStopped();
            break;
        }
    case 9:
        {
            IsNotFiltered = file->IsMovie();
            break;
        }
    case 10:
        {
            IsNotFiltered = file->IsSound();
            break;
        }
    case 11:
        {
            IsNotFiltered = file->IsArchive();
            break;
        }
    case 12:
        {
            IsNotFiltered = file->IsCDImage();
            break;
        }
    case 13:
        {
            IsNotFiltered = file->IsImage();
            break;
        }
    case 14:
        {
            IsNotFiltered = file->IsText();
            break;
        }
    }
    return(IsNotFiltered &&IsInCat);
}

#ifdef __FreeBSD__
long long atoll(char *s)
{
    return strtoll(s, (char **) NULL, 10);
}

#endif /* __FreeBSD__ */

void MakeFoldername(wxString& path)
{
    int length = path.length();
    if (length <= 0)
    {
        return;
    }
    
    if (path.Right(1) == '/')
    {
        path = path.Left(length - 1);
    }
}

void HexDump(const void *buffer, unsigned long buflen)
{
#if defined(__DEBUG__)
    const uint8_t *cbuf = (const uint8_t *) buffer;
    for (unsigned long ofs = 0 ; ofs < buflen ; /* no increment here */
    )
    {
        cout << setfill('0') << setw(8) << hex << ofs;
        for (unsigned long i = 0 ; (i < 8) && (ofs < buflen) ; i++, ofs++)
        {
            cout << setfill('0') << setw(2) << hex << (int) cbuf[ofs];
        }
        cout << endl;
    }
#endif
}

bool BackupFile(const wxString &filename, const wxString &appendix)
{
    wxRemoveFile(filename + appendix);
    if (!wxRenameFile(filename, filename + appendix))
    {
        if (!SafeCopyFile(filename, filename + appendix))
        {
            cout << "info: Could not create backup of '" << filename << "'" << endl;
            return false;
        }
        else if(!wxRemoveFile(filename))
        {
            cout << "info: Could not delete original '" << filename << "' after creating backup" << endl;
        }
    }
    return true;
}

// This function is a modified form of wxWindows-2.4.2's wxCopyFile()
// It's primary purpose is to ignore chmod()s on FAT32 partitions
bool SafeCopyFile(const wxString &file1, const wxString &file2)
{
    wxFile fileIn(file1, wxFile::read);
    if (!fileIn.IsOpened())
    {
        //		return false;// for copying it the file could be open too!!
    }
    wxFile fileOut(file2, wxFile::write);
    char buf[4096];
    size_t count;
    while (true)
    {
        count = fileIn.Read(buf, WXSIZEOF(buf));
        if (fileIn.Error())
        {
            return false;
        }
        if (!count)
        {
            break;
        }
        if (fileOut.Write(buf, count) < count)
        {
            return false;
        }
    }
    if (!fileIn.Close() || !fileOut.Close())
    {
        return false;
    }
    return true;
}
